/*
 * CONTRAT DE LICENCE DE LOGICIEL LIBRE CeCILL-B
 * https://cecill.info/licences/Licence_CeCILL-B_V1-fr.html
 */
package fr.unistra.sil.erp.back.controller.api;

import static fr.unistra.sil.erp.back.WebMvcConfig.MAPPING_RETRIEVEALL;
import fr.unistra.sil.erp.back.DatabaseSystem;
import fr.unistra.sil.erp.back.controller.IRetrieveInfoController;
import fr.unistra.sil.erp.back.model.Item;
import fr.unistra.sil.erp.back.db.DatabaseConnectionException;
import fr.unistra.sil.erp.back.db.DatabaseInterface;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * REST controller for the item list.
 * @author BEAUVAIS ANTOINE
 */
@RestController
public class ApiRetrieveItemsController implements IRetrieveInfoController {
    
    /**
     * Returns all products as a JSON response.
     * @param cat an optional category.
     * @return a JSON response.
     * @throws ApiServerErrorException if the request could not be served.
     * @throws ApiBadRequestException if the category parameter is invalid.
     */
    @GetMapping(MAPPING_RETRIEVEALL)
    public ResponseEntity<Object> retrieveInfo(
            @RequestParam(value = "category", defaultValue = "") String cat
        ) throws ApiServerErrorException, ApiBadRequestException
    {
        DatabaseInterface db;
        try {
            db = DatabaseSystem.getInstance();
        } catch (DatabaseConnectionException ex) {
            Logger.getLogger(ApiRetrieveItemsController.class.getName()).log(
                    Level.SEVERE, "Could not connect to database.", ex);
            throw new ApiServerErrorException("Database failure.");
        }
        
        List<Item> res;
        if(cat.equals(""))
            res = db.getAllItems();
        else
        {
            int category;
            try
            {
                category = Integer.parseInt(cat);
            }
            catch(NumberFormatException ex)
            {
                throw new ApiBadRequestException("Category must be a integer.");
            }
            
            res = db.getItemsFromCategory(category);
        }
        
        if(res == null)
            throw new ApiServerErrorException("Failed to query info.");
        else if(res.isEmpty())
            return ResponseEntity.noContent().build();
        
        return new ResponseEntity<>(res, HttpStatus.OK);
    }
}
